
#include <QTime>
#include <string>
#include <sstream>
#include <iostream>

static const int numValidCharList = 85;
static const QString dummyString = "{{{{";

std::string encodePassword(const std::string str)
{
  QByteArray ba;
  QString sPass = ":";
  QString sTmp;

  if (str.empty())
  {
    return "";
  }

  QString p = QString::fromLocal8Bit(str.c_str());

  for (int i = 0; i < p.length(); i++)
  {
    int c = p.at(i).unicode();

    sTmp.sprintf("%d:", (c + i + 1));
    sPass += sTmp;
    sTmp = "";
  }

  ba = sPass.toLocal8Bit();

  return std::string(ba.constData());
}

static char validCharList[numValidCharList] =
{
  '!', '#', '$', '%', '&', '(', ')', '*', '+', '-',
  '.', '0', '1', '2', '3', '4', '5', '6', '7', '8',
  '9', ':', ';', '<', '>', '?', '@', 'A', 'B', 'C',
  'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M',
  'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W',
  'X', 'Y', 'Z', '[', ']', '_', 'a', 'b', 'c', 'd',
  'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n',
  'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x',
  'y', 'z', '{', '|', '}'
};

static int findCharInList(char c)
{
  int i = -1;

  for (int j = 0; j < numValidCharList; j++)
  {
    if (validCharList[j] == c)
    {
      i = j;

      return i;
    }
  }

  return i;
}

static char getRandomValidCharFromList()
{
  int k = QTime::currentTime().second();

  return validCharList[k];
}

std::string scrambleString(const std::string& input)
{
  QByteArray ba;
  QString sRet;

  if (input.empty())
  {
    return "";
  }

  std::string encoded = encodePassword(input);

  QString str = QString(encoded.c_str());

  if (str.length() < 32)
  {
    str += dummyString;
  }

  for (int iR = (str.length() - 1); iR >= 0; iR--)
  {
    //
    // Reverse string
    //

    sRet += str.at(iR);
  }

  if (sRet.length() < 32)
  {
    sRet += dummyString;
  }

  int k = getRandomValidCharFromList();
  int l = k + sRet.length() - 2;

  sRet.insert(0, k);

  for (int i1 = 1; i1 < (int)sRet.length(); i1++)
  {
    int j = findCharInList(sRet.at(i1).toLatin1());

    if (j == -1)
    {
      return input;
    }

    int i = (j + l * (i1 + 1)) % numValidCharList;

    sRet[i1] = validCharList[i];
  }

  char c = getRandomValidCharFromList() + 2;

  sRet.append(c);

  ba = sRet.toLocal8Bit();

  return std::string(ba.constData());
}


int main (int argc, char *argv[]) {
  if (argc < 1) return 1;
  
  std::string s = scrambleString(argv[1]);
  
  for (size_t i=0; i < s.size(); ++i) {
    char c = s[i];
    switch (c) {
      case '\'': std::cout << "&#039;"; break;
      case '"':  std::cout << "&quot;"; break;
      case '<':  std::cout << "&lt;"; break;
      case '>':  std::cout << "&gt;"; break;
      case '&':  std::cout << "&amp;"; break;
      default: std::cout << c;
    }
  }
  
  return 0;
}

